@@ -269,7 +269,6 @@ static pj_bool_t options_on_rx_request(pjsip_rx_data *rdata)
 {
 	RAII_VAR(struct ast_sip_endpoint *, endpoint, NULL, ao2_cleanup);
 	pjsip_uri *ruri;
-	pjsip_sip_uri *sip_ruri;
 	char exten[AST_MAX_EXTENSION];
 
 	if (pjsip_method_cmp(&rdata->msg_info.msg->line.req.method, &pjsip_options_method)) {
@@ -281,13 +280,12 @@ static pj_bool_t options_on_rx_request(pjsip_rx_data *rdata)
 	}
 
 	ruri = rdata->msg_info.msg->line.req.uri;
-	if (!PJSIP_URI_SCHEME_IS_SIP(ruri) && !PJSIP_URI_SCHEME_IS_SIPS(ruri)) {
+	if (!ast_sip_is_allowed_uri(ruri)) {
 		send_options_response(rdata, 416);
 		return PJ_TRUE;
 	}
 
-	sip_ruri = pjsip_uri_get_uri(ruri);
-	ast_copy_pj_str(exten, &sip_ruri->user, sizeof(exten));
+	ast_copy_pj_str(exten, ast_sip_pjsip_uri_get_username(ruri), sizeof(exten));
 
 	/*
 	 * We may want to match in the dialplan without any user
@@ -350,6 +348,8 @@ static void sip_contact_status_dtor(void *obj)
 {
 	struct ast_sip_contact_status *contact_status = obj;
 
+	ast_sip_security_mechanisms_vector_destroy(&contact_status->security_mechanisms);
+
 	ast_string_field_free_memory(contact_status);
 }
 
@@ -367,6 +367,7 @@ static struct ast_sip_contact_status *sip_contact_status_alloc(const char *name)
 		ao2_ref(contact_status, -1);
 		return NULL;
 	}
+	AST_VECTOR_INIT(&contact_status->security_mechanisms, 0);
 	strcpy(contact_status->name, name); /* SAFE */
 	return contact_status;
 }
@@ -387,6 +388,8 @@ static struct ast_sip_contact_status *sip_contact_status_copy(const struct ast_s
 	dst->rtt = src->rtt;
 	dst->status = src->status;
 	dst->last_status = src->last_status;
+
+	ast_sip_security_mechanisms_vector_copy(&dst->security_mechanisms, &src->security_mechanisms);
 	return dst;
 }
 
@@ -806,7 +809,7 @@ static void qualify_contact_cb(void *token, pjsip_event *e)
 
 	if (ast_sip_push_task(contact_callback_data->aor_options->serializer,
 		sip_options_contact_status_notify_task, contact_callback_data)) {
-		ast_log(LOG_NOTICE, "Unable to queue contact status update for '%s' on AOR '%s', state will be incorrect\n",
+		ast_log(LOG_WARNING, "Unable to queue contact status update for '%s' on AOR '%s', state will be incorrect\n",
 			ast_sorcery_object_get_id(contact_callback_data->contact),
 			contact_callback_data->aor_options->name);
 		ao2_ref(contact_callback_data, -1);
@@ -2722,6 +2725,7 @@ int ast_sip_format_contact_ami(void *obj, void *arg, int flags)
 	struct ast_sip_contact_status *status;
 	struct ast_str *buf;
 	const struct ast_sip_endpoint *endpoint = ami->arg;
+	char secs[AST_TIME_T_LEN];
 
 	buf = ast_sip_create_ami_event("ContactStatusDetail", ami);
 	if (!buf) {
@@ -2733,7 +2737,8 @@ int ast_sip_format_contact_ami(void *obj, void *arg, int flags)
 	ast_str_append(&buf, 0, "AOR: %s\r\n", wrapper->aor_id);
 	ast_str_append(&buf, 0, "URI: %s\r\n", contact->uri);
 	ast_str_append(&buf, 0, "UserAgent: %s\r\n", contact->user_agent);
-	ast_str_append(&buf, 0, "RegExpire: %ld\r\n", contact->expiration_time.tv_sec);
+	ast_time_t_to_string(contact->expiration_time.tv_sec, secs, sizeof(secs));
+	ast_str_append(&buf, 0, "RegExpire: %s\r\n", secs);
 	if (!ast_strlen_zero(contact->via_addr)) {
 		ast_str_append(&buf, 0, "ViaAddress: %s", contact->via_addr);
 		if (contact->via_port) {
